"use strict";
var option_1 = require("./option");
exports.Ok = function (value) { return new Result(value, null, true); };
exports.Err = function (error) { return new Result(null, error, false); };
var Result = (function () {
    function Result(value, error, hasOk) {
        this.value = value;
        this.error = error;
        this.hasOk = hasOk;
    }
    Object.defineProperty(Result.prototype, "ok", {
        get: function () {
            return this.hasOk
                ? option_1.Some(this.value)
                : option_1.None();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Result.prototype, "err", {
        get: function () {
            return !this.hasOk
                ? option_1.Some(this.error)
                : option_1.None();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Result.prototype, "isOk", {
        get: function () {
            return this.hasOk;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Result.prototype, "isErr", {
        get: function () {
            return !this.hasOk;
        },
        enumerable: true,
        configurable: true
    });
    Result.prototype.map = function (cb) {
        return this.hasOk
            ? exports.Ok(cb(this.value))
            : this;
    };
    Result.prototype.mapErr = function (cb) {
        return this.hasOk
            ? this
            : exports.Err(cb(this.error));
    };
    Result.prototype.match = function (matcher) {
        return this.hasOk
            ? matcher.Ok(this.value)
            : matcher.Err(this.error);
    };
    Result.prototype.and = function (res) {
        return this.hasOk
            ? res
            : this;
    };
    Result.prototype.andThen = function (cb) {
        return this.hasOk
            ? cb(this.value)
            : this;
    };
    Result.prototype.or = function (res) {
        return this.hasOk
            ? this
            : res;
    };
    Result.prototype.orElse = function (cb) {
        return this.hasOk
            ? this
            : cb(this.error);
    };
    Result.prototype.unwrapOr = function (optb) {
        return this.hasOk
            ? this.value
            : optb;
    };
    Result.prototype.unwrapOrElse = function (cb) {
        return this.hasOk
            ? this.value
            : cb(this.error);
    };
    Result.prototype.unwrap = function () {
        if (this.hasOk) {
            return this.value;
        }
        throw new Error(this.error.toString());
    };
    Result.prototype.unwrapErr = function () {
        if (this.isErr) {
            return this.error;
        }
        throw new Error(this.value.toString());
    };
    Result.prototype.expect = function (msg) {
        if (this.hasOk) {
            return this.value;
        }
        throw new Error(msg + ": " + this.error.toString());
    };
    return Result;
}());
exports.Result = Result;
//# sourceMappingURL=result.js.map